--[[ 
SwitchSpecs 7.1.0.21

SwitchSpecs is a simple addon that allows the user to change between specs and
gear sets with a single button.

Users will need to label thier gear sets as Spec1, Spec2, Spec3 and Spec4 to 
correspond with the respective specialization.

This version requires the user to write a short macro (see below) and requires a
mouse with three buttons to access all three specializations.

Required Macro
/switchspecs
--]]

-- Simple slash command to access addon
SLASH_SWITCHSPECS1, SLASH_SWITCHSPECS2 = '/ss', '/switchspecs';
local SwitchSpecs = {}


-- checkGear() takes a string = EquipmentSet and tries to equip it
-- This is the second and final attempt to equip an item set.
-- If the equipment set cannot be equipped allow the default error
-- warn users that the equipment cannot be equipped.
function SwitchSpecs:checkGear(Spec)
	if type(Spec)~="string" then
	  return false;
	end
  UseEquipmentSet(Spec);
 
end

-- Values for _wait()
local waitTable = {};
local waitFrame = nil;

-- _wait() takes a number and a function with arguments
-- code was provided by WoWWiki wowwiki.wikia.com/wiki/Wait
function SwitchSpecs_wait(delay, func, ...)
	if(type(delay)~="number" or type(func)~="function") then
      return false;
	end
	if(waitFrame == nil) then
      waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
      waitFrame:SetScript("onUpdate",function (self,elapse)
		  local count = #waitTable;
		  local i = 1;
			while(i<=count) do
			  local waitRecord = tremove(waitTable,i);
			  local d = tremove(waitRecord,1);
			  local f = tremove(waitRecord,1);
			  local p = tremove(waitRecord,1);
				if(d>elapse) then
				  tinsert(waitTable,i,{d-elapse,f,p});
				  i = i + 1;
				else
				  count = count - 1;
				  f(unpack(p));
				end
			end
		end);
	end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

-- equipSet() takes a string = EquipmentSet and tries to equip it
-- If unable to equip, wait until character has changed specializations
-- and try to equip EquipmentSet again.
-- Currently this does not suppress the error message
local equipFrame = nil;

function SwitchSpecs:equipSet(Spec)
	if type(Spec)~="string" then
	  return false;
	end
	
  UseEquipmentSet(Spec);										-- Try to equip the specified EquipmentSet
 
	if equipFrame == nil then
	  equipFrame = CreateFrame("Frame", "MyEventFrame");		-- make a frame and call it MyEventFrame
	end
	
  equipFrame:RegisterEvent("EQUIPMENT_SWAP_FINISHED");			-- register the frame for event
  equipFrame:SetScript("OnEvent",function (self, event, ...)
	  local didSwitch, setName = ...;									-- Collect arguments
		if not didSwitch then											-- If failed to equipped Set
		  SwitchSpecs_wait(5.6, SwitchSpecs.checkGear, SwitchSpec, Spec);				-- Try again after specialization switch complete
		end
		Spec = nil;
	end )
 
end


function SlashCmdList.SWITCHSPECS(msg, editbox)
  
  local btn = GetMouseButtonClicked();
  local equipSets = {};
	
	for i=1, GetNumSpecializations() do 					-- Get Specialization names
	  local _,name=GetSpecializationInfo(i); 
	  equipSets[i] = name; 
	end
	
	for i=GetNumSpecializations()+1, 6 do					-- Create places for alternate equipment sets8
	  equipSets[i] = "Alt"..i-GetNumSpecializations();
--	  print(equipSets[i]);
	end
--[[
	for i=1, table.getn(equipSets) do									-- Error testing for 7.1.0.21
		if (GetEquipmentSetInfoByName(equipSets[i]) == nil) then
		  print("Equipment set "..equipSets[i].." does not exists.");
		  
		else
		  local icon, setID, isEquipped, numItems, numEquipped, unknown, numMissing, numIgnored = GetEquipmentSetInfoByName(equipSets[i]);
		  print("Set name = "..equipSets[i]..", Unknown = "..unknown);
		end
	end
--]]	
	if InCombatLockdown() then 								-- If in combat
--	  print("In Combat, Spec = " .. GetSpecialization());										-- Give a warning that the user is in combat
--	  UseEquipmentSet("Spec" .. GetSpecialization());			-- Attempt to equip any gear (e.g. weapon) that may be equiped
	
	elseif (IsShiftKeyDown() and btn == "MiddleButton") then	-- If user selected fourth spec function
		if (GetEquipmentSetInfoByName(equipSets[6]) == nil) then	-- Check if the Equipment Set has been created
		  print("Equipment set \""..equipSets[6].."\" does not exists.");
		else
		  SwitchSpecs:equipSet(equipSets[6]);							-- Equip item set
		end
		
		if (GetSpecialization() ~=6 and GetNumSpecializations() >5) then						-- If user is not in forth spec
		  SetSpecialization(6);										-- Switch to forth spec	 
		end
	
	elseif (IsShiftKeyDown() and btn == "RightButton") then	-- If user selected fourth spec function
		if (GetEquipmentSetInfoByName(equipSets[5]) == nil) then	-- Check if the Equipment Set has been created
		  print("Equipment set \""..equipSets[5].."\" does not exists.");
		else
		  SwitchSpecs:equipSet(equipSets[5]);							-- Equip item set
		end
		
		if (GetSpecialization() ~=5 and GetNumSpecializations() >4) then						-- If user is not in forth spec
		  SetSpecialization(5);										-- Switch to forth spec	 
		end
	
	elseif (IsShiftKeyDown() and btn == "LeftButton") then	-- If user selected fourth spec function
		if (GetEquipmentSetInfoByName(equipSets[4]) == nil) then	-- Check if the Equipment Set has been created
		  print("Equipment set \""..equipSets[4].."\" does not exists.");
		else
		  SwitchSpecs:equipSet(equipSets[4]);							-- Equip item set "Spec4"
		end
		
		if (GetSpecialization() ~=4 and GetNumSpecializations() >3) then						-- If user is not in forth spec
		  SetSpecialization(4);										-- Switch to forth spec	 
		end
	
	elseif (btn == "MiddleButton") then						-- Else if user selected third spec function
		if (GetEquipmentSetInfoByName(equipSets[3]) == nil) then	-- Check if the Equipment Set has been created
		  print("Equipment set \""..equipSets[3].."\" does not exists.");
		else
		  SwitchSpecs:equipSet(equipSets[3]);							-- Equip item set "Spec3"
		end
		
		if (GetSpecialization() ~=3 and GetNumSpecializations() >2) then						-- If user is not in third spec
		  SetSpecialization(3);										-- Switch to third spec
		end
	
	elseif (btn == "RightButton") then						-- Else if user selected second spec function
		if (GetEquipmentSetInfoByName(equipSets[2]) == nil) then	-- Check if the Equipment Set has been created
		  print("Equipment set \""..equipSets[2].."\" does not exists.");
		else
		  SwitchSpecs:equipSet(equipSets[2]);								-- Equip item set "Spec2"
		end
		
		if (GetSpecialization() ~=2 and GetNumSpecializations() >1) then						-- If user is not in second spec
		  SetSpecialization(2);										-- Switch to second spec		 
		end
	
	elseif (btn == "LeftButton") then						-- Else if user selected first spec function
		if (GetEquipmentSetInfoByName(equipSets[1]) == nil) then	-- Check if the Equipment Set has been created
		  print("Equipment set \""..equipSets[1].."\" does not exists.");
		else
		  SwitchSpecs:equipSet(equipSets[1]);							-- Equip item set "Spec1"
		end
		
		if (GetSpecialization() ~=1) then						-- If user is not in first spec
		  SetSpecialization(1);										-- Switch to first spec
		end
			
	end
	
end